import { jsx as _jsx } from "react/jsx-runtime";
import { createContext, useContext, useEffect, useState, } from "react";
import { AuthenticationDetails, CognitoUser, CognitoUserAttribute, CognitoUserPool, } from "amazon-cognito-identity-js";
const CognitoContext = createContext(undefined);
export function CognitoProvider(props) {
    const [auth, setAuth] = useState({ cognito: props.value });
    useEffect(() => {
        props.value.onChange((cognito) => setAuth({
            cognito,
        }));
        props.value.init()?.catch();
    }, []);
    return (_jsx(CognitoContext.Provider, { value: auth, children: props.children }));
}
export class Cognito {
    pool;
    callbacks = [];
    _session;
    _isInitializing = true;
    get isInitializing() {
        return this._isInitializing;
    }
    set isInitializing(val) {
        this._isInitializing = val;
        this.trigger();
    }
    get session() {
        return this._session;
    }
    set session(session) {
        this._session = session;
        this.trigger();
    }
    get user() {
        return this.pool.getCurrentUser();
    }
    constructor(opts) {
        this.pool = new CognitoUserPool({
            UserPoolId: opts.UserPoolId,
            ClientId: opts.ClientId,
        });
    }
    onChange(cb) {
        this.callbacks.push(cb);
    }
    trigger() {
        for (const cb of this.callbacks) {
            cb(this);
        }
    }
    init() {
        if (!this.user) {
            this.isInitializing = false;
            return Promise.resolve(undefined);
        }
        return new Promise((resolve) => {
            this.user.getSession((err, session) => {
                if (err) {
                    this.user.signOut();
                    resolve(undefined);
                    this.isInitializing = false;
                    return;
                }
                this.session = session;
                resolve(session);
                this.isInitializing = false;
            });
        });
    }
    async login(email, password) {
        const user = new CognitoUser({
            Username: email,
            Pool: this.pool,
        });
        const authDetails = new AuthenticationDetails({
            Username: email,
            Password: password,
        });
        return new Promise((resolve, reject) => {
            user.authenticateUser(authDetails, {
                onSuccess: (result) => {
                    this.session = result;
                    this.trigger();
                    resolve(result);
                },
                onFailure: (err) => {
                    reject(err);
                },
            });
        });
    }
    async register(email, password, attributes = []) {
        return new Promise((resolve, reject) => {
            this.pool.signUp(email, password, [
                ...attributes,
                new CognitoUserAttribute({
                    Name: "email",
                    Value: email,
                }),
            ], [], (err, resp) => {
                if (err) {
                    reject(err);
                    return;
                }
                this.trigger();
                resolve(resp);
            });
        });
    }
    async confirm(username, code) {
        const user = new CognitoUser({
            Username: username,
            Pool: this.pool,
        });
        return new Promise((resolve, reject) => {
            user.confirmRegistration(code, true, (err, result) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve(result);
            });
        });
    }
    async resend(username) {
        const user = new CognitoUser({
            Username: username,
            Pool: this.pool,
        });
        return new Promise((resolve, reject) => {
            user.resendConfirmationCode((err, result) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve(result);
            });
        });
    }
}
export function useCognito() {
    const auth = useContext(CognitoContext);
    return auth.cognito;
}
